@@ -1,7 +1,7 @@
 from powerbot_backtesting.data_acquisition import init_client, get_contract_ids, get_public_trades, get_public_trades_by_days, \
     get_contract_history, get_signals, get_own_trades, get_internal_trades, get_own_orders
 from powerbot_backtesting.data_processing import get_orders, get_ohlc_data, get_orderbooks, calc_trade_vwap, calc_single_vwap
-from powerbot_backtesting.data_analysis import flex_algo, pc_algo, flexpos_algo
+from powerbot_backtesting.data_analysis import flex_algo, pc_algo, flexpos_algo, generate_input_file, BacktestingAlgo
 from powerbot_backtesting.data_visualization import plot_ohlc, ohlc_table, plot_orderbook, plot_volume_history
 from powerbot_backtesting.sql_exporter import PowerBotSQLExporter
 from powerbot_backtesting.historic_data_acquisition import get_historic_data, get_history_key_info
