@@ -2,7 +2,7 @@ import json
 from collections import defaultdict
 from datetime import datetime, timedelta
 from time import sleep
-from typing import Dict, List, Union, Tuple
+from typing import Union
 
 import pandas as pd
 import requests
@@ -27,20 +27,20 @@ def init_client(api_key: str, host: str) -> ApiClient:
 		PowerBot ApiClient Object
 	"""
 	config = Configuration()
-	config.api_key['api_key'] = api_key
+	config.api_key['api_key_security'] = api_key
 	config.host = host
 
 	return ApiClient(config)
 
 
 def get_contract_ids(api_client: ApiClient,
-                     time_from: datetime = None,
-                     time_till: datetime = None,
-                     contract_ids: List[str] = None,
-                     contract_time: str = "all",
-                     products: List[str] = None,
-                     delivery_areas: List[str] = None,
-                     return_contract_objects: bool = False) -> Union[Dict[str, List[str]], Tuple[Dict, Dict]]:
+					 time_from: datetime = None,
+					 time_till: datetime = None,
+					 contract_ids: list[str] = None,
+					 contract_time: str = "all",
+					 products: list[str] = None,
+					 delivery_areas: list[str] = None,
+					 return_contract_objects: bool = False) -> Union[dict[str, list[str]], tuple[dict, dict]]:
 	"""
 	Loads all contract IDs for specified timeframe, either by hours, single day or multiple days. Alternatively, a list
 	of contract IDs can be passed instead of a timeframe, returning a dictionary of contract IDs compatible with all other
@@ -57,14 +57,14 @@ def get_contract_ids(api_client: ApiClient,
 		api_client: PowerBot ApiClient
 		time_from (str/ datetime): yyyy-mm-dd or yyyy-mm-dd hh:mm:ss
 		time_till (str/ datetime): yyyy-mm-dd or yyyy-mm-dd hh:mm:ss
-		contract_ids (List[str]): Optionally, a list of specific contract IDs can be passed to return contract objects
+		contract_ids (list[str]): Optionally, a list of specific contract IDs can be passed to return contract objects
 		contract_time (str): all, hourly, half-hourly or quarter-hourly (all includes UDC)
 		products (list): Optional list of specific products to return
 		delivery_areas (list):
 		return_contract_objects (bool): If True, returns complete Contract object
 
 	Returns:
-		Dict{key: (List[str])}: Dictionary of Contract IDs
+		dict{key: (list[str])}: Dictionary of Contract IDs
 		(Optional) Contract: Contract Object
 	"""
 	# Check date input
@@ -97,8 +97,8 @@ def get_contract_ids(api_client: ApiClient,
 		while time_from != time_till:
 			if contract_time != "all":
 				new_contracts = contract_api.find_contracts(delivery_start=time_from,
-				                                            delivery_end=time_from + timedelta(
-					                                            minutes=timeframes[contract_time]))
+															delivery_end=time_from + timedelta(
+																minutes=timeframes[contract_time]))
 			else:
 				new_contracts = contract_api.find_contracts(delivery_start=time_from)
 
@@ -134,19 +134,19 @@ def get_contract_ids(api_client: ApiClient,
 
 
 def get_public_trades(api_client: ApiClient,
-                      contract_ids: Dict[str, List[str]],
-                      delivery_area: str,
-                      contract_time: str,
-                      iteration_delay: float = 0.4,
-                      serialize_data: bool = True,
-                      add_vwap: bool = False,
-                      use_cached_data: bool = True,
-                      caching: bool = True,
-                      gzip_files: bool = True,
-                      as_csv: bool = False) -> Dict[str, pd.DataFrame]:
+					  contract_ids: dict[str, list[str]],
+					  delivery_area: str,
+					  contract_time: str,
+					  iteration_delay: float = 0.4,
+					  serialize_data: bool = True,
+					  add_vwap: bool = False,
+					  use_cached_data: bool = True,
+					  caching: bool = True,
+					  gzip_files: bool = True,
+					  as_csv: bool = False) -> dict[str, pd.DataFrame]:
 	"""
 	Load trade data for given contract IDs. If add_vwap is True, VWAP will be calculated for each trade, incorporating
-	 all previous trades.
+	all previous trades.
 
 	Args:
 		api_client: PowerBot ApiClient
@@ -162,7 +162,7 @@ def get_public_trades(api_client: ApiClient,
 		as_csv (bool): if True, will save files as CSV, additionally to JSON
 
 	Returns:
-		Dict{key: DataFrame}: Dictionary of DataFrames
+		dict{key: DataFrame}: Dictionary of DataFrames
 	"""
 	# Setup
 	trades = {}
@@ -190,7 +190,8 @@ def get_public_trades(api_client: ApiClient,
 				for i in ["price", "quantity"]:
 					tmp_df[i] = round(tmp_df[i], 2)
 
-				trades[key] = tmp_df
+				# Filter out any contract IDs that are not in value
+				trades[key] = tmp_df.loc[tmp_df.contract_id.isin(value)]
 
 			else:
 				# Save Missing Contract IDs
@@ -215,7 +216,7 @@ def get_public_trades(api_client: ApiClient,
 
 				else:
 					endpoint = f"{api_client.configuration.host}/contract/{item}/{delivery_area}/publictrades?offset={from_public_trade}&limit=500"
-					headers = {"accept": "application/json", "api_key": api_client.configuration.api_key['api_key']}
+					headers = {"accept": "application/json", "api_key_security": api_client.configuration.api_key['api_key_security']}
 					public_trades = json.loads(requests.get(endpoint, headers=headers).text)
 					public_trade_history += public_trades
 
@@ -239,7 +240,7 @@ def get_public_trades(api_client: ApiClient,
 		from powerbot_backtesting.data_processing import calc_trade_vwap
 		for k, v in trades.items():
 			trades[k] = calc_trade_vwap(api_client=api_client, trade_data={k: v}, contract_time=contract_time,
-			                            delivery_area=delivery_area, index="all")
+										delivery_area=delivery_area, index="all")
 
 	if caching:
 		_cache_data("trades", trades, delivery_area, api_client=api_client, gzip_files=gzip_files, as_csv=as_csv)
@@ -247,11 +248,11 @@ def get_public_trades(api_client: ApiClient,
 
 
 def get_public_trades_by_days(api_client: ApiClient,
-                              previous_days: int,
-                              delivery_area: str,
-                              time_from: datetime = None,
-                              contract_time: str = None,
-                              contract_id: str = None) -> Dict[str, pd.DataFrame]:
+							  previous_days: int,
+							  delivery_area: str,
+							  time_from: datetime = None,
+							  contract_time: str = None,
+							  contract_id: str = None) -> dict[str, pd.DataFrame]:
 	"""
 	Gets the contract ID specified by a timeframe or directly by ID and load all trade data for this contract and all
 	contracts in the same timeframe for X previous days.
@@ -265,7 +266,7 @@ def get_public_trades_by_days(api_client: ApiClient,
 		contract_id (str): specific contract ID
 
 	Returns:
-		Dict{key: DataFrame}: Dictionary of DataFrames
+		dict{key: DataFrame}: Dictionary of DataFrames
 	"""
 	if not time_from and not contract_id:
 		raise TypeError("Either time_from and contract_time or a specific contract ID have to be passed.")
@@ -294,14 +295,14 @@ def get_public_trades_by_days(api_client: ApiClient,
 			use_cached_data = False
 
 		contract_ids = get_contract_ids(api_client=api_client, time_from=time_from, time_till=time_till,
-		                                contract_time=contract_time, products=products)
+										contract_time=contract_time, products=products)
 
 		for _ in range(previous_days):
 			time_from -= timedelta(days=1)
 			time_till -= timedelta(days=1)
 
 			contract_ids.update(get_contract_ids(api_client=api_client, time_from=time_from, time_till=time_till,
-			                                     contract_time=contract_time, products=products))
+												 contract_time=contract_time, products=products))
 	except (ValueError, TypeError):
 		raise ValueError("Please use correct format: yyyy-mm-dd hh:mm:ss")
 
@@ -310,14 +311,14 @@ def get_public_trades_by_days(api_client: ApiClient,
 
 
 def get_contract_history(api_client: ApiClient,
-                         contract_ids: Dict[str, List[str]],
-                         delivery_area: str,
-                         iteration_delay: float = 0.4,
-                         serialize_data: bool = True,
-                         use_cached_data: bool = True,
-                         caching: bool = True,
-                         gzip_files: bool = True,
-                         as_csv: bool = False) -> Dict[str, pd.DataFrame]:
+						 contract_ids: dict[str, list[str]],
+						 delivery_area: str,
+						 iteration_delay: float = 0.4,
+						 serialize_data: bool = True,
+						 use_cached_data: bool = True,
+						 caching: bool = True,
+						 gzip_files: bool = True,
+						 as_csv: bool = False) -> dict[str, pd.DataFrame]:
 	"""
 	Load contract history for given contract IDs.
 
@@ -333,7 +334,7 @@ def get_contract_history(api_client: ApiClient,
 		as_csv (bool): if True, will save files as CSV, additionally to JSON
 
 	Returns:
-		Dict{key: DataFrame}: Dictionary of DataFrames
+		dict{key: DataFrame}: Dictionary of DataFrames
 	"""
 	# Setup
 	orders = {}
@@ -350,7 +351,7 @@ def get_contract_history(api_client: ApiClient,
 			for i in [".json.gz", ".json"]:
 				if cache_path.joinpath(f"{filepath}_ordhist{i}").exists():
 					tmp_df = pd.read_json(cache_path.joinpath(f"{filepath}_ordhist{i}"), dtype=False,
-					                      convert_dates=False)
+										  convert_dates=False)
 
 			if isinstance(tmp_df, pd.DataFrame):
 				tmp_df['as_of'] = pd.to_datetime(tmp_df['as_of'])
@@ -359,7 +360,7 @@ def get_contract_history(api_client: ApiClient,
 				cols = {k: v for k, v in cols.items() if k in tmp_df.columns}
 				tmp_df = tmp_df.astype(cols, errors='ignore')
 				for i in ["best_bid_price", "best_bid_quantity", "best_ask_price", "best_ask_quantity", "last_price",
-				          "last_quantity", "total_quantity", "high", "low", "vwap"]:
+						  "last_quantity", "total_quantity", "high", "low", "vwap"]:
 					try:
 						tmp_df[i] = round(tmp_df[i], 2)
 					except (TypeError, KeyError):
@@ -375,15 +376,16 @@ def get_contract_history(api_client: ApiClient,
 										x[param] = round(x[param], 2)
 									try:
 										x["order_entry_time"] = datetime.strptime(x["order_entry_time"],
-										                                          "%Y-%m-%dT%H:%M:%S.%fZ").replace(
+																				  "%Y-%m-%dT%H:%M:%S.%fZ").replace(
 											tzinfo=tzutc())
 									except ValueError:
 										x["order_entry_time"] = datetime.strptime(x["order_entry_time"],
-										                                          "%Y-%m-%dT%H:%M:%SZ").replace(
+																				  "%Y-%m-%dT%H:%M:%SZ").replace(
 											microsecond=0, tzinfo=tzutc())
 					tmp_df["orders"] = order_list
 
-				orders[key] = tmp_df
+				# Filter out any contract IDs that are not in value
+				orders[key] = tmp_df.loc[tmp_df.contract_id.isin(value)]
 
 			else:
 				# Save Missing Contract IDs
@@ -411,7 +413,7 @@ def get_contract_history(api_client: ApiClient,
 
 				else:
 					endpoint = f"{api_client.configuration.host}/contract/{item}/{delivery_area}/history?offset={from_revision}&limit=150&with_owntrades=false&with_signals=false&with_orders=true"
-					headers = {"accept": "application/json", "api_key": api_client.configuration.api_key['api_key']}
+					headers = {"accept": "application/json", "api_key_security": api_client.configuration.api_key['api_key_security']}
 					contract_history = json.loads(requests.get(endpoint, headers=headers).text)
 					public_contract_history += contract_history
 
@@ -437,10 +439,10 @@ def get_contract_history(api_client: ApiClient,
 
 
 def get_signals(api_client: ApiClient,
-                time_from: datetime,
-                time_till: datetime,
-                delivery_area: str = None,
-                portfolio_id: List[str] = None) -> List[Signal]:
+				time_from: datetime,
+				time_till: datetime,
+				delivery_area: str = None,
+				portfolio_id: list[str] = None) -> list[Signal]:
 	"""
 	Function gathers all Signals received by the API in the specified time period and gathers them in a list.
 
@@ -452,24 +454,24 @@ def get_signals(api_client: ApiClient,
 		portfolio_id (str): List of all portfolios that signals should be loaded from
 
 	Returns:
-		List[Signal]
+		list[Signal]
 	"""
 	if not isinstance(time_from, datetime) or not isinstance(time_till, datetime):
 		raise TypeError("time_from and time_till need to be of type datetime")
 
 	return _get_private_data(api_client=api_client,
-	                         data_type="signal",
-	                         time_from=time_from,
-	                         time_till=time_till,
-	                         delivery_area=delivery_area,
-	                         portfolio_id=portfolio_id)
+							 data_type="signal",
+							 time_from=time_from,
+							 time_till=time_till,
+							 delivery_area=delivery_area,
+							 portfolio_id=portfolio_id)
 
 
 def get_own_trades(api_client: ApiClient,
-                   time_from: datetime,
-                   time_till: datetime,
-                   delivery_area: str = None,
-                   portfolio_id: List[str] = None) -> List[Trade]:
+				   time_from: datetime,
+				   time_till: datetime,
+				   delivery_area: str = None,
+				   portfolio_id: list[str] = None) -> list[Trade]:
 	"""
 	Function to collect all Own Trades for the defined time period, either specific to portfolio and/or delivery area
 	or all portfolios and delivery areas used API key has access to.
@@ -479,27 +481,27 @@ def get_own_trades(api_client: ApiClient,
 		time_from (str/ datetime): YYYY-MM-DD or YYYY-MM-DD hh:mm:ss
 		time_till (str/ datetime): YYYY-MM-DD or YYYY-MM-DD hh:mm:ss
 		delivery_area (str): EIC Area Code for Delivery Area
-		portfolio_id (List[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
+		portfolio_id (list[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
 
 	Returns:
-		List[Trade]
+		list[Trade]
 	"""
 	if not isinstance(time_from, datetime) or not isinstance(time_till, datetime):
 		raise TypeError("time_from and time_till need to be of type datetime")
 
 	return _get_private_data(api_client=api_client,
-	                         data_type="own_trade",
-	                         time_from=time_from,
-	                         time_till=time_till,
-	                         delivery_area=delivery_area,
-	                         portfolio_id=portfolio_id)
+							 data_type="own_trade",
+							 time_from=time_from,
+							 time_till=time_till,
+							 delivery_area=delivery_area,
+							 portfolio_id=portfolio_id)
 
 
 def get_internal_trades(api_client: ApiClient,
-                        time_from: datetime,
-                        time_till: datetime,
-                        delivery_area: str = None,
-                        portfolio_id: List[str] = None) -> List[InternalTrade]:
+						time_from: datetime,
+						time_till: datetime,
+						delivery_area: str = None,
+						portfolio_id: list[str] = None) -> list[InternalTrade]:
 	"""
 	Function to collect all Internal Trades for the defined time period, either specific to portfolio and/or delivery
 	area or all portfolios and delivery areas used API key has access to.
@@ -509,26 +511,26 @@ def get_internal_trades(api_client: ApiClient,
 		time_from (str/ datetime): YYYY-MM-DD or YYYY-MM-DD hh:mm:ss
 		time_till (str/ datetime): YYYY-MM-DD or YYYY-MM-DD hh:mm:ss
 		delivery_area (str): EIC Area Code for Delivery Area
-		portfolio_id (List[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
+		portfolio_id (list[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
 
 	Returns:
-		List[InternalTrade]
+		list[InternalTrade]
 	"""
 	if not isinstance(time_from, datetime) or not isinstance(time_till, datetime):
 		raise TypeError("time_from and time_till need to be of type datetime")
 
 	return _get_private_data(api_client=api_client,
-	                         data_type="internal_trade",
-	                         time_from=time_from,
-	                         time_till=time_till,
-	                         delivery_area=delivery_area,
-	                         portfolio_id=portfolio_id)
+							 data_type="internal_trade",
+							 time_from=time_from,
+							 time_till=time_till,
+							 delivery_area=delivery_area,
+							 portfolio_id=portfolio_id)
 
 
 def get_own_orders(api_client: ApiClient,
-                   delivery_area: str = None,
-                   portfolio_id: List[str] = None,
-                   active_only: bool = False) -> List[OwnOrder]:
+				   delivery_area: str = None,
+				   portfolio_id: list[str] = None,
+				   active_only: bool = False) -> list[OwnOrder]:
 	"""
 	Function to collect all available Own Orders, either specific to portfolio and/or delivery area or all portfolios
 	and delivery areas used API key has access to.
@@ -536,14 +538,14 @@ def get_own_orders(api_client: ApiClient,
 	Args:
 		api_client: PowerBot ApiClient
 		delivery_area (str): EIC Area Code for Delivery Area
-		portfolio_id (List[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
+		portfolio_id (list[str]): List of specific portfolio IDs to load trades from. If left out, will load from all IDs.
 		active_only (bool):  True if only active orders should be loaded. If False, loads also hibernate and inactive.
 
 	Returns:
-		List[OwnOrder]
+		list[OwnOrder]
 	"""
 	return _get_private_data(api_client=api_client,
-	                         data_type="own_order",
-	                         delivery_area=delivery_area,
-	                         portfolio_id=portfolio_id,
-	                         active_only=active_only)
+							 data_type="own_order",
+							 delivery_area=delivery_area,
+							 portfolio_id=portfolio_id,
+							 active_only=active_only)
