@@ -2,7 +2,7 @@ import os
 import shutil
 from datetime import timedelta
 from pathlib import Path
-from typing import List, Dict, Union
+from typing import Union
 
 import pandas as pd
 
@@ -10,8 +10,8 @@ from powerbot_backtesting.utils import _cache_data, _find_cache, _historic_data_
 from powerbot_backtesting.utils.constants import *
 
 
-def process_historic_data(extracted_files: Union[List[str], Dict[str, List[str]]],
-                          exchange: str):
+def process_historic_data(extracted_files: Union[list[str], dict[str, list[str]]],
+						  exchange: str):
 	"""
 	Function processes list of files extracted from a zip-file downloaded via History API to be compatible with the
 	rest of the powerbot_backtesting package. Once files have been processed, they are cached in the same manner as
@@ -19,7 +19,7 @@ def process_historic_data(extracted_files: Union[List[str], Dict[str, List[str]]
 	the cache.
 
 	Args:
-		extracted_files (List(str), Dict[str, List[str]]): List of files extracted with get_historic_data
+		extracted_files (list(str), dict[str, list[str]]): List of files extracted with get_historic_data
 		(-> return value of get_historic_data)
 		exchange (str): One of the following: epex, hupx, tge, nordpool, southpool
 	"""
@@ -43,8 +43,7 @@ def process_historic_data(extracted_files: Union[List[str], Dict[str, List[str]]
 
 		while start <= end:
 			for timestep in [15, 30, 60]:
-				ids = df[(df.deliveryStart == start) & (
-						df.deliveryEnd == (start + timedelta(minutes=timestep)))]._id.tolist()
+				ids = df.loc[(df.deliveryStart == start) & (df.deliveryEnd == (start + timedelta(minutes=timestep)))]._id.tolist()
 				if ids:
 					contract_times[
 						f"{start.strftime(DATE_YMD_TIME_HM)} - "
@@ -56,7 +55,7 @@ def process_historic_data(extracted_files: Union[List[str], Dict[str, List[str]]
 		# For each timestep
 		for time, ids in contract_times.items():
 			files = {"trades": [i for i in unzipped_files if any(str(x) in i for x in ids) and "Trades" in i],
-			         "orders": [i for i in unzipped_files if any(str(x) in i for x in ids) and "Orders" in i]}
+					 "orders": [i for i in unzipped_files if any(str(x) in i for x in ids) and "Orders" in i]}
 
 			for k, v in files.items():
 				if v:
