@@ -4,7 +4,7 @@ from datetime import datetime
 from decimal import Decimal, getcontext
 from pathlib import Path
 from time import sleep
-from typing import List, Dict, Union
+from typing import Union
 
 import pandas as pd
 from powerbot_client import ApiClient, TradesApi, SignalsApi, Signal, Trade, InternalTrade, OrdersApi, OwnOrder
@@ -13,12 +13,12 @@ from powerbot_backtesting.utils import *
 
 
 def _get_private_data(api_client: ApiClient,
-                      data_type: str,
-                      time_from: datetime = None,
-                      time_till: datetime = None,
-                      delivery_area: str = None,
-                      portfolio_id: List[str] = None,
-                      active_only: bool = False) -> List[Union[InternalTrade, OwnOrder, Trade, Signal]]:
+					  data_type: str,
+					  time_from: datetime = None,
+					  time_till: datetime = None,
+					  delivery_area: str = None,
+					  portfolio_id: list[str] = None,
+					  active_only: bool = False) -> list[Union[InternalTrade, OwnOrder, Trade, Signal]]:
 	param_mapping = {
 		"internal_trade": {"delivery_within_start": time_from, "delivery_within_end": time_till, "limit": 100},
 		"own_trade": {"delivery_within_start": time_from, "delivery_within_end": time_till, "limit": 500},
@@ -43,8 +43,7 @@ def _get_private_data(api_client: ApiClient,
 		params["delivery_area"] = delivery_area
 
 	while more_obj:
-		new_objs = func_mapping[data_type](offset=offset,
-		                                   **param_mapping[data_type])
+		new_objs = func_mapping[data_type](offset=offset, **param_mapping[data_type])
 		if len(new_objs):
 			coll += new_objs
 			offset += len(new_objs)
@@ -56,16 +55,16 @@ def _get_private_data(api_client: ApiClient,
 
 
 def _cache_data(data_type: str,
-                data: Dict[str, pd.DataFrame],
-                delivery_area: str,
-                exchange: str = None,
-                api_client: object = None,
-                timesteps: int = 0,
-                time_unit: str = None,
-                gzip_files: bool = True,
-                as_json: bool = True,
-                as_csv: bool = False,
-                as_pickle: bool = False):
+				data: dict[str, pd.DataFrame],
+				delivery_area: str,
+				exchange: str = None,
+				api_client: object = None,
+				timesteps: int = 0,
+				time_unit: str = None,
+				gzip_files: bool = True,
+				as_json: bool = True,
+				as_csv: bool = False,
+				as_pickle: bool = False):
 	"""
 	Function to be called by data request functions to cache loaded data in a reusable format. Automatically generates
 	a folder to cache loaded files, if it cannot find an existing one.
@@ -100,47 +99,48 @@ def _cache_data(data_type: str,
 		file_name = f"{file_name}_{data_type}" if folder == "raw" else f"{file_name}_{data_type}_{timesteps}{time_unit}"
 
 		# Check if __cache__ already exists
-		cache_path = _find_cache()
+		cache_path = _find_cache().joinpath(
+			f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\{folder}")
 
 		# Assure That Directory Exists
-		cache_path.joinpath(f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\{folder}").mkdir(
-			parents=True, exist_ok=True)
+		cache_path.mkdir(parents=True, exist_ok=True)
 
 		# Cache File If It Doesn't Exist Yet
-		if as_json and not cache_path.joinpath(
-				f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\"
-				f"{folder}\\{file_name}.json{file_ending}").exists():
-			value.to_json(cache_path.joinpath(
-				f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\{folder}\\"
-				f"{file_name}.json{file_ending}"),
-				date_format="iso", date_unit="us", compression=compression)
+		if as_json and not cache_path.joinpath(f"{file_name}.json{file_ending}").exists():
+			value.to_json(cache_path.joinpath(f"{file_name}.json{file_ending}"), date_format="iso", date_unit="us",
+			              compression=compression)
 
-		if as_csv and not cache_path.joinpath(
-				f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\"
-				f"{folder}\\{file_name}.csv").exists():
-			value.to_csv(cache_path.joinpath(
-				f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\"
-				f"{folder}\\{file_name}.csv"),
-				sep=";", compression=compression)
+		if as_csv and not cache_path.joinpath(f"{file_name}.csv").exists():
+			value.to_csv(cache_path.joinpath(f"{file_name}.csv{file_ending}"), sep=";", compression=compression)
 
-		if as_pickle and not cache_path.joinpath(
-				f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\"
-				f"{folder}\\{file_name}.p").exists():
-			cache_path.joinpath(
-				f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\{folder}").mkdir(
-				parents=True, exist_ok=True)
-			pickle.dump(value,
-			            open(cache_path.joinpath(
-				            f"{environment}\\{exchange}_{delivery_area}\\{year_month}\\{day_month}\\"
-				            f"{folder}\\{file_name}.p"), "wb"))
+		if as_pickle and not cache_path.joinpath(f"{file_name}.p").exists():
+			pickle.dump(value, open(cache_path.joinpath(f"{file_name}.p"), "wb"))
 
 
-def _find_cache():
+def _find_cache() -> Path:
 	"""
 	Functions returns location of __cache__ directory if it can be found within 3 parent directories based on the
-	location of the file backtesting functions are called from. It is assumed that the swagger_client that is necessary
-	for this package is located directly under the project root, therefore the function will stop once it finds this
-	directory. Cache will not be created outside of project directory.
+	location of the file backtesting functions are called from.
+
+	If multiple projects lie within a directory that can be reached within 3 parent directories from where this
+	function is called, it might find a cache directory that does not lie directly in the current projects root
+	directory. This can be confusing, however, it does not restrict functionality.
+
+	This fact can also be used on purpose by manually creating a __pb_cache__ directory one parent above, that can be
+	shared by multiple projects that require historic data.
+
+	Proposed structure:
+	parent_dir
+	| __pb_cache__
+	|
+	|--- project_1
+	|	|   file.py
+	|
+	|--- project_2
+	|	|   file.py
+	|
+	|--- project_3
+		|   file.py
 
 	Returns:
 		Path
@@ -159,10 +159,6 @@ def _find_cache():
 			cache_path = Path(cache_path[0])
 			break
 
-		# Check if project root was reached
-		if "swagger_client" in [x for i in [directory for root, directory, file in os.walk(root_path)] for x in i]:
-			break
-
 		root_path = root_path.parent
 
 	if not cache_path:
@@ -171,7 +167,7 @@ def _find_cache():
 	return cache_path
 
 
-def _get_file_cachepath(api_client, contract_key, delivery_area):
+def _get_file_cachepath(api_client: ApiClient, contract_key: str, delivery_area: str) -> str:
 	"""
 	Helper function that constructs most of the path of a cached file.
 
@@ -192,7 +188,7 @@ def _get_file_cachepath(api_client, contract_key, delivery_area):
 	return f"{environment}\\{market}_{delivery_area}\\{year_month}\\{day_month}\\raw\\{file_name}"
 
 
-def _check_contracts(contract, delivery_areas: List[str], products: List[str]):
+def _check_contracts(contract, delivery_areas: list[str], products: list[str]) -> bool:
 	"""
 	Helper function to determine if contract is of interest and should be added to contract dictionary.
 
@@ -218,13 +214,25 @@ def _check_contracts(contract, delivery_areas: List[str], products: List[str]):
 def _order_matching(order_side: str,
                     orderbook: pd.DataFrame,
                     timestamp: str,
-                    price: int,
-                    quantity: int,
-                    exec_orders_list: Dict[str, int],
-                    trade_list: Dict[int, Dict[str, int]],
-                    contract_time: datetime) -> int:
+                    price: Union[int, float],
+                    quantity: Union[int, float],
+                    exec_orders_list: dict[str, int],
+                    trade_list: dict[int, dict[str, int]],
+                    contract_time: Union[int, float],
+                    order_execution: str = "NON") -> Decimal:
 	"""
-	Matches orders according to input parameters; add trades made to trade_list and returns the remaining quantity.
+	Matches orders according to input parameters; adds trades made to trade_list and returns the remaining quantity.
+
+	The order_execution parameter can be added to decide according to which logic the quantity should be filled. Allowed
+	values are:
+
+	NON - No restriction, partial execution is allowed
+
+	FOK - Fill or Kill, if order isn't filled completely by first matching order, next matching order is loaded ->
+	if none match next order book is loaded
+
+	IOC - Immediate and Cancel, order is executed to maximum extent by first matching order, next orderbook is loaded ->
+	allows price adjustments
 
 	Args:
 		order_side (str): buy/sell
@@ -234,14 +242,15 @@ def _order_matching(order_side: str,
 		quantity (int): Quantity to buy/sell
 		exec_orders_list (dict): Dictionary of already matched order IDs
 		trade_list (list): List of executed trades
-		contract_time (datetime):
+		contract_time (int): contract time in minutes, either 60, 30 or 15
+		order_execution (str): Type of order execution that should be simulated
 
 	Returns:
 		quantity: remaining quantity as int
 	"""
 
 	# Transform Values to Decimals
-	getcontext().prec = 6
+	getcontext().prec = 8
 	price = round(Decimal(price), 2)
 	quantity = round(Decimal(quantity), 1)
 	cash_adjust = {60: 1, 30: 2, 15: 4}
@@ -249,7 +258,7 @@ def _order_matching(order_side: str,
 	order_type = {"buy": "ask", "sell": "bid"}
 	operator = {"buy": -1, "sell": 1}
 
-	orderbook = orderbook[orderbook.type == order_type[order_side]]
+	orderbook = orderbook.loc[orderbook.type == order_type[order_side]]
 
 	if order_type[order_side] == "ask":
 		orderbook = orderbook.sort_values(by=['price', 'as_of'], ascending=[True, False])
@@ -269,15 +278,19 @@ def _order_matching(order_side: str,
 			price_match = Decimal(row["price"]) <= price if order_side == "buy" else Decimal(row["price"]) >= price
 
 			if price_match:
+				# If order can't be filled completely
+				if order_execution == "FOK" and quantity > open_qty:
+					continue
+
 				traded_quant = round(min(open_qty, quantity), 1)
-				cash = round(traded_quant * Decimal(row.price) * operator[order_side] / cash_adjust[contract_time],
-				             2)  # Calculate Cost
+				# Calculate Cost
+				cash = traded_quant * Decimal(row.price) * operator[order_side] / cash_adjust[contract_time]
 
 				trade_list[len([*trade_list]) + 1] = {"Side": order_side,
-				                                      "Quantity": float(str(traded_quant)),
-				                                      "Price": row["price"],
-				                                      "Cash": float(str(cash)),
-				                                      "Timestamp": timestamp}
+													  "Quantity": float(str(traded_quant)),
+													  "Price": row["price"],
+													  "Cash": float(str(cash)),
+													  "Timestamp": timestamp}
 
 				if row["order_id"] in [*exec_orders_list]:
 					# If Existing, Adjust Quantity
@@ -289,6 +302,11 @@ def _order_matching(order_side: str,
 					exec_orders_list[row["order_id"]] = round(min(open_qty, quantity), 1)
 
 				quantity -= round(traded_quant, 1)  # Adjust Quantity
+
+				# Break if quantity has been executed
+				if order_execution == "IOC":
+					break
+
 			else:
 				break
 		else:
@@ -297,9 +315,9 @@ def _order_matching(order_side: str,
 	return quantity
 
 
-def _orderbook_data_transformation(orders: pd.DataFrame):
+def _orderbook_data_transformation(orders: pd.DataFrame) -> pd.DataFrame:
 	"""
-	Function transforms data in passes dataframe to be compatible with process_orderbooks function
+	Function transforms data in passed dataframe to be compatible with process_orderbooks function
 
 	Returns:
 		Dataframe
@@ -312,36 +330,63 @@ def _orderbook_data_transformation(orders: pd.DataFrame):
 	if "orders" in orders.columns:
 		orders_all = orders["orders"].to_list()
 		dates_all = [str(i) for i in orders["as_of"].to_list()]
+		deltas = orders.delta.to_list() if "delta" in orders.columns else [1 for _ in range(len(dates_all))]
 		for nr, val in enumerate(orders_all):
 			for k, v in val.items():
-				if v:
-					if k == "bid":
-						for x in v:
-							x["as_of"] = dates_all[nr]
-							x["type"] = "bid"
-							bids_asks.append(x)
-					if k == "ask":
-						for x in v:
-							x["as_of"] = dates_all[nr]
-							x["type"] = "ask"
-							bids_asks.append(x)
+				if v and k in ["ask", "bid"]:
+					for x in v:
+						x["as_of"] = dates_all[nr]
+						x["type"] = "bid" if k == "bid" else "ask"
+						x["delta"] = val.get("delta", deltas[nr])
+						bids_asks.append(x)
 
 	else:
 		for nr, row in orders.iterrows():
 			for side in ["bids", "asks"]:
-				if row[side]:
+				if row[side] and not isinstance(row[side], float):
 					for entry in row[side]:
-						entry["type"] = side
+						entry["type"] = side[:-1]
 						entry["as_of"] = row["as_of"]
+						entry["delta"] = row["delta"]
 						bids_asks.append(entry)
 
 	df_bid_asks = pd.DataFrame(bids_asks)
 	df_bid_asks = df_bid_asks.drop(columns=["exe_restriction", "delivery_area", "order_entry_time"],
-	                               errors="ignore")
+								   errors="ignore")
 	return df_bid_asks
 
 
-def _historic_data_transformation(files, exchange, filetype):
+def _delta_filter(orderbook: pd.DataFrame) -> pd.DataFrame:
+	"""
+	Function filters dataframe by orders that are not delta reports. If delta is False, all orders before this order
+	have to be deleted.
+
+	Since delta: false is assigned to a revision of a contract, it can contain more than just one order. Therefore, all
+	orders in a delta: false revision have that flag assigned. This function takes this situation in account, loading
+	the last delta: false and going back until the space between two orders that have delta: false is bigger than 1.
+
+	Args:
+		orderbook: pd.Dataframe
+
+	Returns:
+		pd.Dataframe
+	"""
+	ind = orderbook[(~orderbook.delta) | (orderbook.delta == 0)].index
+	if not ind.empty:
+		last_delta = ind[-1]
+
+		for i in ind[::-1]:
+			if i == (last_delta - 1) or i == last_delta:
+				last_delta = i
+			else:
+				break
+
+		return orderbook.loc[orderbook.index >= last_delta].drop(columns=["delta"])
+
+	return orderbook.drop(columns=["delta"])
+
+
+def _historic_data_transformation(files: list, exchange: str, filetype: str) -> pd.DataFrame:
 	df = None
 	for file in files:
 		if not isinstance(df, pd.DataFrame):
@@ -353,10 +398,10 @@ def _historic_data_transformation(files, exchange, filetype):
 		if filetype == "trades":
 			df.drop(columns=["revisionNo"], inplace=True)
 			df.rename(columns={"_id": "trade_id", "contractId": 'contract_id', "tradeExecTime": 'exec_time',
-			                   "apiTimeStamp": 'api_timestamp', "buyDeliveryArea": 'buy_delivery_area',
-			                   "sellDeliveryArea": 'sell_delivery_area', "selfTrade": 'self_trade',
-			                   "qty": 'quantity', "px": 'price', "pxqty": 'prc_x_qty'},
-			          inplace=True)
+							   "apiTimeStamp": 'api_timestamp', "buyDeliveryArea": 'buy_delivery_area',
+							   "sellDeliveryArea": 'sell_delivery_area', "selfTrade": 'self_trade',
+							   "qty": 'quantity', "px": 'price', "pxqty": 'prc_x_qty'},
+					  inplace=True)
 
 			df.quantity = df.quantity / 100
 			df.price = df.price / 100
@@ -364,16 +409,16 @@ def _historic_data_transformation(files, exchange, filetype):
 
 		if filetype == "orders":
 			df.rename(columns={"asOf": "as_of", "bestAskPrice": "best_ask", "bestAskQuantity": "best_ask_qty",
-			                   "bestBidPrice": "best_bid", "bestBidQuantity": "best_bid_qty",
-			                   "contractId": "contract_id", "deliveryArea": "delivery_area", "full": "delta",
-			                   "lastPrice": "last_price", "lastQuantity": "last_qty",
-			                   "lastUpdate": "last_trade_time", "revisionNo": "revision_no"},
-			          inplace=True)
+							   "bestBidPrice": "best_bid", "bestBidQuantity": "best_bid_qty",
+							   "contractId": "contract_id", "deliveryArea": "delivery_area", "full": "delta",
+							   "lastPrice": "last_price", "lastQuantity": "last_qty",
+							   "lastUpdate": "last_trade_time", "revisionNo": "revision_no"},
+					  inplace=True)
 
 			# Getting information from details field if missing on upper level
 			meta_cols = {"best_bid": "bestBidPx", "best_ask": "bestAskPx", "best_bid_qty": "bestBidQty",
-			             "last_price": "lastPx", "last_qty": "lastQty", "last_trade_time": "lastTradeTime",
-			             "volume": "totalQty", "high": "highPx", "low": "lowPx"}
+						 "last_price": "lastPx", "last_qty": "lastQty", "last_trade_time": "lastTradeTime",
+						 "volume": "totalQty", "high": "highPx", "low": "lowPx"}
 			missing_details = {k: v for k, v in meta_cols.items() if k not in df.columns}
 			details = df.details.tolist()
 
@@ -389,8 +434,8 @@ def _historic_data_transformation(files, exchange, filetype):
 			bids = [i["buyOrdrList"]["ordrBookEntry"] if i["buyOrdrList"] else None for i in df.details.tolist()]
 
 			conversion = lambda i: [{"order_id": v["ordrId"], "price": v["px"] / 100, "quantity": v["qty"] / 100,
-			                         "contract_id": contract_id, "delivery_area": delivery_area,
-			                         "order_entry_time": v["ordrEntryTime"]} for v in i]
+									 "contract_id": contract_id, "delivery_area": delivery_area,
+									 "order_entry_time": v["ordrEntryTime"]} for v in i]
 
 			asks = [conversion(i) if i else None for i in asks]
 			bids = [conversion(i) if i else None for i in bids]
@@ -403,20 +448,20 @@ def _historic_data_transformation(files, exchange, filetype):
 	else:
 		if filetype == "trades":
 			# Filter out deleted Trades
-			df = df[df.deleted != True]
+			df = df.loc[~df.deleted]
 
 			df.rename(columns={"_id": "trade_id", "tradeTime": 'exec_time', "apiTimestamp": "api_timestamp",
-			                   "companyTrade": "self_trade"},
-			          inplace=True)
+							   "companyTrade": "self_trade"},
+					  inplace=True)
 
 			details = df.legs.tolist()
 
 			df["contract_id"] = [i[0]["contractId"] for i in details]
 			df["buy_delivery_area"] = [NORDPOOL_EIC_CODES[i[0]["deliveryAreaId"]] if i else None for i in
-			                           details]
+									   details]
 			df["sell_delivery_area"] = [NORDPOOL_EIC_CODES[i[1]["deliveryAreaId"]] if len(i) == 2 else None for
-			                            i in
-			                            details]
+										i in
+										details]
 			df["quantity"] = [float(i[0]["quantity"]) for i in details]
 			df["price"] = [float(i[0]["unitPrice"]) for i in details]
 			df["prc_x_qty"] = round(df.price * df.quantity, 2)
@@ -426,11 +471,11 @@ def _historic_data_transformation(files, exchange, filetype):
 		if filetype == "orders":
 			df.rename(
 				columns={"apiTimestamp": "as_of", "bestAskPrice": "best_ask", "bestAskQuantity": "best_ask_qty",
-				         "bestBidPrice": "best_bid", "bestBidQuantity": "best_bid_qty", "bidsAndAsks": "details",
-				         "contractId": "contract_id", "deliveryArea": "delivery_area", "full": "delta",
-				         "lastPrice": "last_price", "lastQuantity": "last_qty", "lowestPrice": "low",
-				         "highestPrice": "high", "lastTradeTime": "last_trade_time", "revision": "revision_no",
-				         "turnover": "total_quantity"},
+						 "bestBidPrice": "best_bid", "bestBidQuantity": "best_bid_qty", "bidsAndAsks": "details",
+						 "contractId": "contract_id", "deliveryArea": "delivery_area", "full": "delta",
+						 "lastPrice": "last_price", "lastQuantity": "last_qty", "lowestPrice": "low",
+						 "highestPrice": "high", "lastTradeTime": "last_trade_time", "revision": "revision_no",
+						 "turnover": "total_quantity"},
 				inplace=True)
 
 			df.revision_no = [i for i in range(0, len(df))]
@@ -448,4 +493,4 @@ def _historic_data_transformation(files, exchange, filetype):
 
 			df.drop(columns=["_id", "details", "updatedAt"], inplace=True)
 
-	return df
+	return df
\ No newline at end of file
